package com.bull.cimero.pluginEditor.editors.model.BCModel;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.spagic.utils.SpagicLogger;

/**
 * This class is the RSS model.
 * It contain the properties of a RSS flow
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class RSS extends BindingComponentModel {

	/** The id serialize. */
	private static final long serialVersionUID = 8140366783698510696L;
	//
	//CONSTANT
	//
	/**The default name of a RSS component.*/
	private static final String DEFAULTNAME = "myRSSFlow";
	/**The default polled frequence.*/
	private static final int DEFAULTFREQUENCE = 5000;
	//
	//PROPERTIES
	//
	/**The urls id.*/
	private static final String URLS = "urls";
	/**The flow type id.*/
	private static final String FLOWTYPE = "flowType";
	/**The polled frequence id.*/
	private static final String POLLFREQUENCE = "polledfrequence";
	/**The last read date id.*/
	private static final String LASTREADDATE = "lastreaddate";
	/**The destination name id.*/
	private static final String NAMEDEST = "destination";
	/**The rss path id.*/
	private static final String RSSPATH = "rssPath";
	/**The rss title id.*/
	private static final String RSSTITLE = "title";
	/**The rss body id.*/
	private static final String RSSXPATHBODY = "body";
	/**The id of this element.*/
	private static final String ID = "Rss";
	//
	//variables
	//
	/**The date of the rss flow.*/
	private DateFormat dateFormat = null;

	/**
	 * The default constructor.
	 */
	public RSS() {
		super(DEFAULTNAME);
		if (setInput(false)) {
			setOutput(true);
		}
		//init of dateformat
		Locale locale = Locale.US;
		/**
		 * DateFormat build with format :
		 * SHORT = 01/01/2002
		 * FULL = lundi 1 janvier 2002
		 */
		this.dateFormat = new SimpleDateFormat("yyyy,MMM,dd" , locale);
		//set the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(URLS, "");
		setProperty(INOUT, "");
		setProperty(FLOWTYPE, "rss_2.0");
		setProperty(POLLFREQUENCE, String.valueOf(DEFAULTFREQUENCE));
		setProperty(LASTREADDATE, null);
		setProperty(NAMEDEST, "");
		setProperty(RSSPATH, "");
		setProperty(RSSTITLE, "");
		setProperty(RSSXPATHBODY, "");
		setProperty(LASTREADDATE, dateFormat.format(new Date()));
	}

	/**
	 * This function is used to determine if this element is valide.
	 * @param showLog : true if the log must to be show.
	 * @return true is this element is valide.
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		if (getName().compareTo("") == 0) {
			msg += "\n- Name => the element name cannot be null";
			returnResult = false;
		}
		if (isOutput()) {
			//mode in
			if (getUrls().compareTo("") == 0) {
				msg += "\n- Urls => At least one rss file path must "
					+ "to be specify";
				returnResult = false;
			}
			if (getFlowType().compareTo("") == 0) {
				msg += "\n- FlowType => the flow type is necessary. "
					+ "The default value is 'rss_2.0'";
				returnResult = false;
			}
			if (getReadFrequence() < 0) {
				msg += "\n- Read frequence => the read frequence "
					+ "cannot be a negative number";
				returnResult = false;
			}
			if (getNameFirstDestination().compareTo("") == 0) {
				msg += "\n- Destination => the destination service "
					+ "cannot be null";
				returnResult = false;
			}
		} else {
			//mode out
			if (getRssPath().compareTo("") == 0) {
				msg += "\n- Rss file adress => At least one rss file "
					+ "must to be specify";
				returnResult = false;
			}
			if (getTitle().compareTo("") == 0) {
				msg += "\n- Title => The title to the rss messages "
					+ "cannot be null";
				returnResult = false;
			}
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				SpagicLogger.warning(msg);
			}
		}
		return returnResult;
	}

	/**
	 * Create the descriptor to the property views.
	 */
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, indice++ + " - name : ");
		addComboBoxDescriptor(INOUT,
	    		indice++ + " - mode", new String[]{"IN", "OUT"});
		if (isOutput()) {
			addTextDescriptor(URLS, indice++ + " - rss adress : ");
			addTextDescriptor(FLOWTYPE, indice++ + " - rss version : ");
			addTextDescriptor(POLLFREQUENCE, indice++
					+ " - polled frequence: ");
			addTextDescriptor(LASTREADDATE, indice++ + " - last read date : ");
			String[] lst = getLstObjectDrawWithInputConnection();
			addComboBoxDescriptor(NAMEDEST, indice++
					+ " - destination : ", lst);
		} else {
			addTextDescriptor(RSSPATH, indice++ + " - rss file adress : ");
			addTextDescriptor(RSSTITLE, indice++ + " - title : ");
			addTextDescriptor(RSSXPATHBODY, indice++ + " - body : ");
		}
	}

	/**
	 * This function is used to set the variable. It is an abstract method
	 * of IpropertySource.
	 * @param id : the id of the object
	 * @param value : the value of this object
	 */
	public final void setPropertyValue(final Object id, Object value) {
		if (id == INOUT) {
			if (isInput()) {
				if (setInput(false)) {
					setOutput(true);
				}
			} else {
				if (setOutput(false)) {
					setInput(true);
				}
			}
		} else if (id == POLLFREQUENCE) {
			int oldFrequence = getReadFrequence();
			try {
				Integer.parseInt((String) value);
			} catch (Exception e) {
				SpagicLogger.error("Error in format ! "
						+ "The last value will be restore ", e);
				value = String.valueOf(oldFrequence);
			}
		} else if (id == LASTREADDATE) {
			String oldDate = getLastPollDate();
			try {
				dateFormat.parse((String) value);
			} catch (Exception e) {
				SpagicLogger.error("Error in format ! "
						+ "The last value will be restore ", e);
				value = oldDate;
			}
		} else if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				SpagicLogger.error("problem when created connection",
						new Exception());
			}
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * This function is the implementation of an abstract method of
	 * IPropertySource class.
	 * It is used to return the value of selected element identified by id.
	 * @param id : the id of the element
	 * @return Object : the value of this element
	 */
	public final Object getPropertyValue(final Object id) {
		if (INOUT.equals(id)) {
			if (!isInput()) {
				return 0;
			} else {
				return 1;
			}
		} else if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
		} else {
			return super.getPropertyValue(id);
		}
		return null;
	}

	/**
	 * Return the last polled date.
	 * @return Date
	 */
	public final String getLastPollDate() {
		try {
			return dateFormat.format(
					dateFormat.parse((String) getProperty(LASTREADDATE)));
		} catch (ParseException e) {
			SpagicLogger.error("error during parse date", e);
			return dateFormat.format(new Date());
		}
	}

	/**
	 * This function is the implementation of an abstract method of
	 * IPropertySource class.
	 * It is used to reset the value of all property.
	 * @param id : the id of the selected element
	 */
	public void resetPropertyValue(final Object id) {

	}

	/**
	 * get the body.
	 * @return Returns the body.
	 */
	public final String getBody() {
		return (String) getProperty(RSSXPATHBODY);
	}

	/**
	 * get the type.
	 * @return Returns the flowType.
	 */
	public final String getFlowType() {
		return (String) getProperty(FLOWTYPE);
	}

	/**
	 * get the read frequence.
	 * @return Returns the readFrequence.
	 */
	public final int getReadFrequence() {
		return Integer.parseInt((String) getProperty(POLLFREQUENCE));
	}

	/**
	 * get the path.
	 * @return Returns the rssPath.
	 */
	public final String getRssPath() {
		return (String) getProperty(RSSPATH);
	}

	/**
	 * get the title.
	 * @return Returns the title.
	 */
	public final String getTitle() {
		return (String) getProperty(RSSTITLE);
	}

	/**
	 * get the date last polled.
	 * @return Returns the urls.
	 */
	public final String getUrls() {
		return (String) getProperty(URLS);
	}

	/**
	 * get the if.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}

	/**
	 * set the date last polled.
	 * @param dateLastPolledParam The dateLastPolled to set.
	 */
	public final void setDateLastPolled(final Date dateLastPolledParam) {
		setProperty(LASTREADDATE, dateFormat.format(dateLastPolledParam));
	}

	/**
	 * set the type of the flow.
	 * @param flowTypeParam The flowType to set.
	 */
	public final void setFlowType(final String flowTypeParam) {
		setProperty(FLOWTYPE, flowTypeParam);
	}

	/**
	 * set the readfrequence.
	 * @param readFrequenceParam The readFrequence to set.
	 */
	public final void setReadFrequence(final int readFrequenceParam) {
		setProperty(POLLFREQUENCE, String.valueOf(readFrequenceParam));
	}

	/**
	 * set the path.
	 * @param rssPathParam The rssPath to set.
	 */
	public final void setRssPath(final String rssPathParam) {
		setProperty(RSSPATH, rssPathParam);
	}


	/**
	 * set the title.
	 * @param titleParam The title to set.
	 */
	public final void setTitle(final String titleParam) {
		setProperty(RSSTITLE, titleParam);
	}

	/**
	 * set the urls.
	 * @param urlsParam The urls to set.
	 */
	public final void setUrls(final String urlsParam) {
		setProperty(URLS, urlsParam);
	}
}
