package com.bull.cimero.pluginEditor.utils.propertyCombo;

import java.text.MessageFormat;
import org.eclipse.jface.util.Assert;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

/**
 * A cell editor that presents a list of items in a combo box.
 * The cell editor's value is the zero-based index of the selected
 * item.
 * This class may be instantiated; it is not intended to be subclassed.
 * @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public class ComboBoxCellEditorEditable extends CellEditor {

	/**the number of passage in dogetvalue function.*/
	private int passage = 0;
	/**The oldvalue specify by the user.*/
	private String oldValue = "";
	/** The list of items to present in the combo box. */
    private String[] items;
    /** The zero-based index of the selected item. */
    private int selection;
    /** The custom combo box control. */
    private CCombo comboBox;
    /**The comboboxPorpertyEditable that call this class.*/
	private ComboBoxPropertyEditable myCombo;
    /** Default ComboBoxCellEditor style. */
    private static final int DEFAULTSTYLE = SWT.NONE;
    /** layout minimum width. */
	public static final int LAYOUTMINIMUMWIDTH = 60;
    /** font minimum width. */
	public static final int FONTMINIMUMWIDTH = 10;

    /**
     * Default constructor.
     * Creates a new cell editor with no control and no  st of choices.
     * Initially, the cell editor has no cell validator.
     * @since 2.1
     * @see CellEditor#setStyle
     * @see CellEditor#create
     * @see ComboBoxCellEditor#setItems
     * @see CellEditor#dispose
     */
    public ComboBoxCellEditorEditable() {
        setStyle(DEFAULTSTYLE);
    }

    /**
     * Creates a new cell editor with a combo containing the given
     * list of choices and parented under the given control. The cell
     * editor value is the zero-based index of the selected item.
     * Initially, the cell editor has no cell validator and
     * the first item in the list is selected.
     *
     * @param parent the parent control
     * @param itemsParam the list of strings for the combo box
     * @param comboProp : properties
     */
    public ComboBoxCellEditorEditable(
    		final Composite parent
    		, final String[] itemsParam
    		, final ComboBoxPropertyEditable comboProp) {
        this(parent, itemsParam, DEFAULTSTYLE, comboProp);
    }

    /**
     * Creates a new cell editor with a combo containing the given
     * list of choices and parented under the given control. The cell
     * editor value is the zero-based index of the selected item.
     * Initially, the cell editor has no cell validator and
     * the first item in the list is selected.
     *
     * @param parent the parent control
     * @param itemsParam the list of strings for the combo box
     * @param style the style bits
     * @param comboProp : properties
     * @since 2.1
     */
    public ComboBoxCellEditorEditable(
    		final Composite parent
    		, final String[] itemsParam, final int style
    		, final ComboBoxPropertyEditable comboProp) {
        super(parent, style);
        this.myCombo = comboProp;
        setItems(itemsParam);
    }

    /**
     * Returns the list of choices for the combo box.
     * @return the list of choices for the combo box
     */
    public final String[] getItems() {
        return this.items;
    }

    /**
     * Sets the list of choices for the combo box.
     * @param itemsParam the list of choices for the combo box
     */
    public final void setItems(final String[] itemsParam) {
        Assert.isNotNull(itemsParam);
        this.items = itemsParam;
        populateComboBoxItems();
        myCombo.setLabels(itemsParam);
    }

    /**
     * Method declared on CellEditor.
     * @param parent : the parent that call this function.
     * @return the control created.
     */
    protected final Control createControl(final Composite parent) {

        comboBox = new CCombo(parent, getStyle());
        comboBox.setFont(parent.getFont());

        comboBox.addKeyListener(new KeyAdapter() {
            // hook key pressed - see PR 14201
            public void keyPressed(final KeyEvent e) {
                keyReleaseOccured(e);
            }
        });

        comboBox.addSelectionListener(new SelectionAdapter() {
            public void widgetDefaultSelected(final SelectionEvent event) {
                applyEditorValueAndDeactivate();
            }

            public void widgetSelected(final SelectionEvent event) {
                selection = comboBox.getSelectionIndex();
            }
        });

        comboBox.addTraverseListener(new TraverseListener() {
            public void keyTraversed(final TraverseEvent e) {
                if (e.detail == SWT.TRAVERSE_ESCAPE
                        || e.detail == SWT.TRAVERSE_RETURN) {
                    e.doit = false;
                }
            }
        });

        comboBox.addFocusListener(new FocusAdapter() {
            public void focusLost(final FocusEvent e) {
            	ComboBoxCellEditorEditable.this.focusLost();
            }
        });
        return comboBox;
    }

    /**
     * The <code>ComboBoxCellEditor</code> implementation of
     * this <code>CellEditor</code> framework method returns
     * the zero-based index of the current selection.
     * @return the zero-based index of the current selection wrapped
     * as an <code>Integer</code>
     */
    protected final Object doGetValue() {
    	//if the text value doesn't exist
    	String value = comboBox.getText();
    	if ((passage != 0) || (value.compareTo("") == 0)) {
    		value = oldValue;
    		passage = 0;
    	} else {
    		passage++;
    	}
    	if (!textExist(value)) {
    		//add an items.
    		String[] itemsTmp = new String [items.length + 1];
    		for (int i = 0; i < items.length; i++) {
    			itemsTmp[i] = items[i];
    		}
    		itemsTmp[items.length] = value;
    		setItems(itemsTmp);
    		populateComboBoxItems();
    	}
    	this.oldValue = value;
    	//get the element index.
    	int index = getElementIndex(value);
        return new Integer(index);
    }

    /**
     * get an element index.
     * @param value : the string to search.
     * @return the index of the element pass in parameter.
     */
    private int getElementIndex(final String value) {
    	for (int i = 0; i < comboBox.getItemCount(); i++) {
    		if (comboBox.getItem(i).compareTo(value) == 0) {
    			return i;
    		}
    	}
    	return -1;
	}

	/**
     * to know if a string already exist in the items combo box list.
     * @param text : the string to test
     * @return true if the text already exist.
     */
    private boolean textExist(final String text) {
    	for (int i = 0; i < comboBox.getItemCount(); i++) {
    		if (comboBox.getItem(i).compareTo(text) == 0) {
    			return true;
    		}
    	}
    	return false;
	}

	/**
     * Method declared on CellEditor.
     */
    protected final void doSetFocus() {
        comboBox.setFocus();
    }

    /**
     * The <code>ComboBoxCellEditor</code> implementation of
     * this <code>CellEditor</code> framework method sets the
     * minimum width of the cell.  The minimum width is 10 characters
     * if <code>comboBox</code> is not <code>null</code> or
     * <code>disposed</code> eles it is 60 pixels to make sure the
     * arrow button and some text is visible.
     * The list of CCombo will be wide enough to show its longest item.
     * @return LayoutData the layout
     */
    public final LayoutData getLayoutData() {
        LayoutData layoutData = super.getLayoutData();
        if ((comboBox == null) || comboBox.isDisposed()) {
			layoutData.minimumWidth = LAYOUTMINIMUMWIDTH;
		} else {
            // make the comboBox 10 characters wide
            GC gc = new GC(comboBox);
            layoutData.minimumWidth = (gc.getFontMetrics()
                    .getAverageCharWidth() * FONTMINIMUMWIDTH)
                    + FONTMINIMUMWIDTH;
            gc.dispose();
        }
        return layoutData;
    }

    /**
     * The <code>ComboBoxCellEditor</code> implementation of
     * this <code>CellEditor</code> framework method
     * accepts a zero-based index of a selection.
     *
     * @param value the zero-based index of the selection wrapped
     *   as an <code>Integer</code>
     */
    protected final void doSetValue(final Object value) {
        Assert.isTrue(comboBox != null && (value instanceof Integer));
        selection = ((Integer) value).intValue();
        comboBox.select(selection);
        myCombo.setLabels(items);
    }

    /**
     * Updates the list of choices for the combo box for the current control.
     */
    private void populateComboBoxItems() {
        if (comboBox != null && items != null) {
            comboBox.removeAll();
            for (int i = 0; i < items.length; i++) {
				comboBox.add(items[i], i);
			}

            setValueValid(true);
            selection = 0;
        }
        myCombo.setLabels(items);
    }

    /**
     * Applies the currently selected value and
     * deactivates the cell editor.
     */
    public final void applyEditorValueAndDeactivate() {
        //must set the selection before getting value
        selection = comboBox.getSelectionIndex();
        Object newValue = doGetValue();
        markDirty();
        boolean isValid = isCorrect(newValue);
        setValueValid(isValid);

        if (!isValid) {
        	//Only format if the 'index' is valid
        	if (items.length > 0
        			&& selection >= 0
        			&& selection < items.length) {
	            //try to insert the current value into the error message.
	            setErrorMessage(MessageFormat.format(getErrorMessage(),
	                    new Object[] { items[selection] }));
        	} else {
	            //Since we don't have a valid index,
        		//assume we're using an 'edit'
        		// combo so format using its text value
	            setErrorMessage(MessageFormat.format(getErrorMessage(),
	                    new Object[] { comboBox.getText() }));
        	}
        }
        myCombo.setLabels(items);
        fireApplyEditorValue();
        deactivate();
    }

    /**
     *
     * @see org.eclipse.jface.viewers.CellEditor#focusLost()
     */
    protected final void focusLost() {
    	myCombo.setLabels(items);
        if (isActivated()) {
            applyEditorValueAndDeactivate();
        }
    }

    /**
     * event occured : key released.
     * @see org.eclipse.jface.viewers.CellEditor#keyReleaseOccured
     * (org.eclipse.swt.events.KeyEvent)
     * @param keyEvent : event
     */
    protected final void keyReleaseOccured(final KeyEvent keyEvent) {
        if (keyEvent.character == '\u001b') { // Escape character
            fireCancelEditor();
        } else if (keyEvent.character == '\t') { // tab key
            applyEditorValueAndDeactivate();
        }
    }
}
